@@ -1,10 +1,24 @@
 """
 Implementation of the XDG Menu Specification Version 1.0.draft-1
 http://standards.freedesktop.org/menu-spec/
+
+Example code:
+
+from xdg.Menu import parse, Menu, MenuEntry
+
+def print_menu(menu, tab=0):
+  for submenu in menu.Entries:
+    if isinstance(submenu, Menu):
+      print ("\t" * tab) + unicode(submenu)
+      print_menu(submenu, tab+1)
+    elif isinstance(submenu, MenuEntry):
+      print ("\t" * tab) + unicode(submenu.DesktopEntry)
+
+print_menu(parse())
 """
 
-from __future__ import generators
 import locale, os, xml.dom.minidom
+import subprocess
 
 from xdg.BaseDirectory import *
 from xdg.DesktopEntry import *
@@ -15,14 +29,11 @@ import xdg.Config
 
 ELEMENT_NODE = xml.dom.Node.ELEMENT_NODE
 
-# for python <= 2.3
-try:
-    reversed = reversed
-except NameError:
-    def reversed(x):
-        return x[::-1]
-
 class Menu:
+    """Menu containing sub menus under menu.Entries
+
+	Contains both Menu and MenuEntry items.
+    """
     def __init__(self):
         # Public stuff
         self.Name = ""
@@ -90,15 +101,27 @@ class Menu:
     # FIXME: Performance: cache getName()
     def __cmp__(self, other):
         return locale.strcoll(self.getName(), other.getName())
+    
+    def _key(self):
+        """Key function for locale-aware sorting."""
+        return locale.strxfrm(self.getName())
+    
+    def __lt__(self, other):
+        try:
+            other = other._key()
+        except AttributeError:
+            pass
+        return self._key() < other
 
     def __eq__(self, other):
-        if self.Name == str(other):
-            return True
-        else:
-            return False
+        try:
+            return self.Name == unicode(other)
+        except NameError:  # unicode() becomes str() in Python 3
+            return self.Name == str(other)
 
     """ PUBLIC STUFF """
     def getEntries(self, hidden=False):
+        """Interator for a list of Entries visible to the user."""
         for entry in self.Entries:
             if hidden == True:
                 yield entry
@@ -110,6 +133,7 @@ class Menu:
     # return multiple items
 
     def getMenuEntry(self, desktopfileid, deep = False):
+        """Searches for a MenuEntry with a given DesktopFileID."""
         for menuentry in self.MenuEntries:
             if menuentry.DesktopFileID == desktopfileid:
                 return menuentry
@@ -118,6 +142,7 @@ class Menu:
                 submenu.getMenuEntry(desktopfileid, deep)
 
     def getMenu(self, path):
+        """Searches for a Menu with a given path."""
         array = path.split("/", 1)
         for submenu in self.Submenus:
             if submenu.Name == array[0]:
@@ -127,6 +152,7 @@ class Menu:
                     return submenu
 
     def getPath(self, org=False, toplevel=False):
+        """Returns this menu's path in the menu structure."""
         parent = self
         names=[]
         while 1:
@@ -147,24 +173,28 @@ class Menu:
         return path
 
     def getName(self):
+        """Returns the menu's localised name."""
         try:
             return self.Directory.DesktopEntry.getName()
         except AttributeError:
             return self.Name
 
     def getGenericName(self):
+        """Returns the menu's generic name."""
         try:
             return self.Directory.DesktopEntry.getGenericName()
         except AttributeError:
             return ""
 
     def getComment(self):
+        """Returns the menu's comment text."""
         try:
             return self.Directory.DesktopEntry.getComment()
         except AttributeError:
             return ""
 
     def getIcon(self):
+        """Returns the menu's icon, filename or simple name"""
         try:
             return self.Directory.DesktopEntry.getIcon()
         except AttributeError:
@@ -292,6 +322,19 @@ class Rule:
 
     def __str__(self):
         return self.Rule
+    
+    def do(self, menuentries, type, run):
+        for menuentry in menuentries:
+            if run == 2 and ( menuentry.MatchedInclude == True \
+            or menuentry.Allocated == True ):
+                continue
+            elif eval(self.Rule):
+                if type == "Include":
+                    menuentry.Add = True
+                    menuentry.MatchedInclude = True
+                else:
+                    menuentry.Add = False
+        return menuentries
 
     def compile(self):
         exec("""
@@ -412,7 +455,7 @@ class MenuEntry:
         return self.DesktopEntry.filename.replace(self.Filename, '')
 
     def getType(self):
-        # Can be one of System/User/Both
+        """Return the type of MenuEntry, System/User/Both"""
         if xdg.Config.root_mode == False:
             if self.Original:
                 return "Both"
@@ -451,6 +494,18 @@ class MenuEntry:
 
     def __cmp__(self, other):
         return locale.strcoll(self.DesktopEntry.getName(), other.DesktopEntry.getName())
+    
+    def _key(self):
+        """Key function for locale-aware sorting."""
+        return locale.strxfrm(self.DesktopEntry.getName())
+    
+    def __lt__(self, other):
+        try:
+            other = other._key()
+        except AttributeError:
+            pass
+        return self._key() < other
+            
 
     def __eq__(self, other):
         if self.DesktopFileID == str(other):
@@ -498,11 +553,12 @@ def parse(filename=None):
         filename = __getFileName(filename)
 
     # use default if no filename given
+    if not filename: 
+        candidate = os.environ.get('XDG_MENU_PREFIX', '') + "applications.menu"
+        filename = __getFileName(candidate)
+        
     if not filename:
-        filename = __getFileName("applications.menu")
-
-    if not filename:
-        raise ParsingError('File not found', "/etc/xdg/menus/applications.menu")
+        raise ParsingError('File not found', "/etc/xdg/menus/%s" % candidate)
 
     # check if it is a .menu file
     if not os.path.splitext(filename)[1] == ".menu":
@@ -841,8 +897,8 @@ def __mergeLegacyDir(dir, prefix, filename, parent):
         return m
 
 def __parseKDELegacyDirs(filename, parent):
-    f=os.popen3("kde-config --path apps")
-    output = f[1].readlines()
+    output = subprocess.check_output(['kde-config', '--path', 'apps'],
+                                     universal_newlines=True).splitlines()
     try:
         for dir in output[0].split(":"):
             __parseLegacyDir(dir,"kde", filename, parent)
@@ -1024,7 +1080,7 @@ class MenuEntryCache:
 
     def addMenuEntries(self, dirs, prefix="", legacy=False):
         for dir in dirs:
-            if not self.cacheEntries.has_key(dir):
+            if not dir in self.cacheEntries:
                 self.cacheEntries[dir] = []
                 self.__addFiles(dir, "", prefix, legacy)
 
